import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/routes/verb_selection_route.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_view.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// The Route that allows you to show the evaluation of the indicative of the
/// [Verb] that you are studying.
class VerbIndicativeEvaluateRoute extends StatelessWidget {
  static const routeName = '/verbEvaluateRoute';

  final SettingsController _settingsController;

  const VerbIndicativeEvaluateRoute(this._settingsController, {Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: _settingsController,
      builder: (BuildContext context, Widget? child) {
        return MaterialApp(
          debugShowCheckedModeBanner: false,
          themeMode: _settingsController.themeMode,
          theme: ThemeData(),
          darkTheme: ThemeData.dark(),
          home: SafeArea(
            child: Scaffold(
              appBar: AppBar(
                actions: [
                  _repeatIcon(context),
                  _homeIcon(context),
                  _settingsIcon(context)
                ],
                title: _titleCell(context),
              ),
              body: _centerWidget(context),
            ),
          ),
        );
      },
    );
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.restorablePushNamed(context, SettingsView.routeName);
        });
  }

  IconButton _homeIcon(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: const Key('menu_key'),
        onPressed: () {
          testModel.reset();
          Navigator.restorablePopAndPushNamed(
              context, VerbSelectionRoute.routeName);
        },
        icon: const Icon(Icons.home));
  }

  IconButton _repeatIcon(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return IconButton(
        key: const Key('repeat_key'),
        onPressed: () {
          testModel.reset();
          Navigator.popAndPushNamed(
              context, VerbStudymodeSelectionRoute.routeName);
        },
        icon: const Icon(Icons.refresh));
  }

  Wrap _titleCell(BuildContext context) {
    final VerbTestModel testModel =
        Provider.of<VerbTestModel>(context, listen: false);
    return Wrap(direction: Axis.horizontal, children: [
      const Text(
        "Vervoeging ",
      ),
      Text(
        testModel.infinitive,
        textScaleFactor: TextSizeLimiter.textScaleFactor(context),
      ),
    ]);
  }

  Widget _centerWidget(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: UniqueKey(),
        body: _bodyListView(context),
      ),
    );
  }

  Padding _scoreWidget(BuildContext context) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return Padding(
      padding: const EdgeInsets.fromLTRB(20, 5, 20, 5),
      child: Card(
          elevation: 10,
          child: ListTile(
            title: Container(
              padding: const EdgeInsets.fromLTRB(5, 5, 5, 5),
              child: Text(
                "Score:",
                style: const TextStyle(
                  color: Colors.blue,
                  fontSize: 25,
                ),
                textScaleFactor: TextSizeLimiter.textScaleFactor(context),
              ),
            ),
            trailing: Container(
              padding: const EdgeInsets.fromLTRB(75, 5, 30, 5),
              child: Text(
                "${model.score}/${model.items.length}",
                style: const TextStyle(
                  color: Colors.blue,
                  fontSize: 25,
                ),
                textScaleFactor: TextSizeLimiter.textScaleFactor(context),
              ),
            ),
          )),
    );
  }

  Future<void> _showCorrectAnswerDialog(
      String expected, BuildContext context) async {
    return showDialog<void>(
        context: context,
        barrierDismissible: true,
        builder: (BuildContext context) {
          return AlertDialog(
            title: Text(
              "Juiste antwoord",
              key: UniqueKey(),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            content: Text(
              expected,
              key: UniqueKey(),
              style: const TextStyle(
                color: Colors.green,
                fontSize: 25,
              ),
              textScaleFactor: TextSizeLimiter.textScaleFactor(context),
            ),
            actions: [
              TextButton(
                key: const Key('got_it_button'),
                onPressed: () {
                  Navigator.of(context).pop();
                },
                child: Text(
                  "Oké. Ik snap het.",
                  key: UniqueKey(),
                  textScaleFactor: TextSizeLimiter.textScaleFactor(context),
                ),
              )
            ],
          );
        });
  }

  Widget _pronounCell(BuildContext context, person) {
    VerbTestModel model = context.watch<VerbTestModel>();
    return PronounField(
        targetLanguage: model.targetLanguageIso,
        fullPronoun: model.pronouns[person],
        expectedIndicative: model.expected[person]);
  }

  Widget _resultCell(BuildContext context, person) {
    VerbTestModel model = context.watch<VerbTestModel>();
    String actual = model.items[person];
    String expected = model.expected[person];
    return Row(children: [
      _pronounCell(context, person),
      Expanded(
        key: UniqueKey(),
        child: Text(
          actual,
          key: UniqueKey(),
          textAlign: TextAlign.right,
          style: TextStyle(
            color: _actualTextColor(actual, expected),
            fontSize: 25,
          ),
          textScaleFactor: TextSizeLimiter.textScaleFactor(context),
        ),
      ),
      Visibility(
        child: SizedBox(
          width: MediaQuery.of(context).size.width * 0.07,
          child: IconButton(
              key: wrongAnswerKey(expected, person),
              onPressed: () {
                _showCorrectAnswerDialog(expected, context);
              },
              icon: const Icon(
                Icons.help_outline_sharp,
              )),
        ),
        visible: (actual != expected),
      )
    ]);
  }

  Color _actualTextColor(String actual, String expected) {
    return (actual == expected) ? Colors.green : Colors.red;
  }

  ListView _bodyListView(BuildContext context) {
    return ListView(
      key: UniqueKey(),
      scrollDirection: Axis.vertical,
      children: _bodyListTiles(context),
    );
  }

  List<Widget> _bodyListTiles(BuildContext context) {
    List<Widget> _widgets = [];

    _widgets.add(_scoreWidget(context));
    _widgets.add(const Divider());

    List<Widget> _rows = [];
    for (int i = 0; i < _itemCount(context); i++) {
      _rows.add(Card(
        key: UniqueKey(),
        elevation: 10,
        child: Padding(
          key: UniqueKey(),
          padding: const EdgeInsets.all(5.0),
          child: _resultCell(context, i),
        ),
      ));
    }

    _widgets.add(Padding(
      key: UniqueKey(),
      padding: const EdgeInsets.fromLTRB(20, 0, 20, 0),
      child: Card(
        key: UniqueKey(),
        child: Column(
          key: UniqueKey(),
          children: _rows,
        ),
      ),
    ));

    return _widgets;
  }

  int _itemCount(BuildContext context) {
    return Provider.of<VerbTestModel>(context, listen: false).items.length;
  }

  static Key wrongAnswerKey(String expected, int person) {
    return Key('wrong_${expected}_$person');
  }
}
